package internal

import (
	"codeup.aliyun.com/6552e56cc3b2728a4557fc18/plugin/proto"
	"encoding/json"
	"plugins/union_pay/internal/po"
	"plugins/union_pay/internal/utils"
	"plugins/union_pay/internal/vo"
	"strings"
)

type Config struct {
	Config *proto.Config
	Extra  *ConfigExtra
}

type ConfigExtra struct {
	ChNlId string `json:"chnlId"`  // 渠道方代码
	IV     string `json:"iv"`      // 加密密钥
	KEY    string `json:"key"`     // 加密密钥
	RsaPrk string `json:"rsa_prk"` // 私钥
	RsaNpk string `json:"rsa_npk"` // 回调公钥
}

func transConfig(config *proto.Config) (*Config, error) {
	var extra ConfigExtra
	err := json.Unmarshal(config.Extra, &extra)
	if err != nil {
		return nil, err
	}
	return &Config{
		Config: config,
		Extra:  &extra,
	}, nil
}

func orderReq(in *proto.OrderRequest, config *Config) (*po.OrderReq, error) {
	type OrderExtra struct {
		OrderDt string `json:"orderDt"`
	}
	var e OrderExtra
	_ = json.Unmarshal(in.Order.Extra, &e)

	mobile, err := utils.Encrypt([]byte(in.Order.Account), []byte(config.Extra.KEY), []byte(config.Extra.IV))
	if err != nil {
		return nil, err
	}
	return &po.OrderReq{
		Cmd:      vo.OrderCmd,
		AccessId: vo.AccessId,
		EntityTp: vo.EntityTp,

		ChNlId:      config.Extra.ChNlId,
		Qid:         in.Order.OrderNo,
		OrderDt:     e.OrderDt,
		TraceId:     in.Order.OrderNo,
		DiscountId:  in.Product.ProductNo,
		Mobile:      mobile,
		DiscountNum: in.Order.Quantity,
	}, nil
}

func orderResp(request *proto.OrderRequest, resp po.OrderResp) *proto.OrderResponse {
	data, _ := json.Marshal(resp)
	result := &proto.Result{
		OrderNo: request.Order.OrderNo,
		TradeNo: resp.CouponCd,
		Message: resp.Msg,
		Data:    data,
	}
	if !resp.Code.IsSuccess() {
		result.Status = proto.Status_FAIL
		result.Message = resp.GetMsg()
	} else {
		result.Status = proto.Status_ING
	}
	return &proto.OrderResponse{Result: result}
}

func queryReq(in *proto.QueryRequest, chNlId string) *po.QueryReq {
	type OrderExtra struct {
		OrigDate string `json:"origDate"`
	}
	var e OrderExtra
	_ = json.Unmarshal(in.Order.Extra, &e)
	return &po.QueryReq{
		ChNlId:   chNlId,
		Cmd:      vo.QueryCmd,
		OrigQid:  in.Order.OrderNo,
		OrigDate: e.OrigDate,
		TraceId:  in.Order.OrderNo,
	}
}

func queryResp(request *proto.QueryRequest, resp po.QueryResp) *proto.QueryResponse {
	data, _ := json.Marshal(resp)
	result := &proto.Result{
		OrderNo: request.Order.OrderNo,
		TradeNo: resp.CouponCd,
		Status:  resp.OperaSt.GetOrderStatus(),
		Message: resp.GetMsg(),
		Data:    data,
	}
	return &proto.QueryResponse{Result: result}
}

func notifyReq(in *proto.NotifyRequest) *po.Notify {
	var h po.Headers
	_ = json.Unmarshal(in.Headers, &h)
	var body po.Body
	_ = json.NewDecoder(strings.NewReader(string(in.Body))).Decode(&body)
	body.CouponNum = utils.ConvertToInt(body.CouponNum)
	body.OrderAt = utils.ConvertToInt(body.OrderAt)
	body.DiscountAt = utils.ConvertToInt(body.DiscountAt)
	for i, cdInfo := range body.CouponCdInfos {
		cdInfo.SubAcctOperAt = utils.ConvertToInt(cdInfo.SubAcctOperAt)
		body.CouponCdInfos[i] = cdInfo
	}
	return &po.Notify{
		Headers: &h,
		Body:    &body,
	}
}

func notifyResp(request *proto.NotifyRequest, n *po.Notify) *proto.NotifyResponse {
	result := &proto.Result{
		OrderNo: n.Body.TransSeq,
		TradeNo: n.Body.CouponCd,
		Status:  n.Body.OperTp.GetOrderStatus(),
		Message: n.Body.OperTp.GetText(),
		Data:    request.Body,
	}
	return &proto.NotifyResponse{Result: result}
}
